Emo = LibStub("AceAddon-3.0"):NewAddon("Emo", "AceConsole-3.0", "AceEvent-3.0")

local db
local defaults = {
	profile = {
		button_click = "ALL",
		display_show = true,
		display_unlocked = false,
		display_scale = 32,
		display_alpha = 1,
		display_icon = 1,
		display_tooltip = true,
	}
}

---------------- ON INITIALIZE ----------------

function Emo:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EmoDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db = self.db.profile

	self:CreateFrames()
	self:SetupOptions()

	self:RegisterChatCommand("emo", "ShowConfig")
end

function Emo:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
	Emo:Reset()
end


---- FRAME FUNCTIONS ----

local function startmoving(self)
	if not db.display_unlocked then
		Emo.Frame.IsMovingOrSizing = 1
		Emo.Frame:StartMoving()
	end
end

local function stopmoving(self)
	if Emo.Frame.IsMovingOrSizing then
		Emo.Frame:StopMovingOrSizing()
		db.display_x, db.display_y = Emo.Frame:GetLeft(), Emo.Frame:GetBottom()
		Emo.Frame.IsMovingOrSizing = nil
	end
end

function Emo:CreateFrames()

	self.Frame = CreateFrame("Button","EmoButton",UIParent)
	self.Frame:SetMovable(true)
	self.Frame:EnableMouse(true)
	self.Frame:RegisterForDrag("LeftButton")
	self.Frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	self.Frame:SetFrameStrata("MEDIUM")

	local t = self.Frame:CreateTexture(nil,"BACKGROUND")
	t:SetAllPoints(self.Frame)
	self.Frame.texture = t

	Emo:Icon()
	Emo:SetPosition()
	Emo:Scale()
	Emo:Alpha()

	self.Frame:Show()

	self.Frame:SetScript("OnDragStart", startmoving)
	self.Frame:SetScript("OnDragStop", stopmoving)
	self.Frame:SetScript("OnEnter", function() if db.display_tooltip then Emo:Tooltip() end end)
	self.Frame:SetScript("OnLeave", function() if db.display_tooltip then GameTooltip:Hide() end end)
	self.Frame:SetScript("OnClick", function(self, button, down)
		if button == "RightButton" then
			ToggleDropDownMenu(1, nil, EmoMenu2, self:GetName(), 0, 0)
		elseif (db.button_click == "ALL") or (db.button_click == nil) then
			ToggleDropDownMenu(1, nil, EmoMenu1, self:GetName(), 0, 0)
		else
			Emo:DoEmote(db.button_click)
		end
	end)

end

function Emo:OnEnable()
	db.display_show = true,
	self.Frame:Show()
end

function Emo:OnDisable()
	db.display_false = true,
	self.Frame:Hide()
end

function Emo:Show()
	if not db.display_show then
		self.Frame:Hide()
	else
		self.Frame:Show()
	end
end

function Emo:Scale()
	self.Frame:SetWidth(db.display_scale)
	self.Frame:SetHeight(db.display_scale)
end

function Emo:Alpha()
	self.Frame:SetAlpha(db.display_alpha)
end

function Emo:Icon()
	local icon
	if db.display_icon == 1 then icon = "Interface\\ChatFrame\\UI-ChatIcon-Chat-Up" 
	elseif db.display_icon == 2 then icon = "Interface\\Icons\\Ability_Rogue_Disguise" 
	elseif db.display_icon == 3 then icon = "Interface\\Icons\\INV_Mask_01"
	elseif db.display_icon == 4 then icon = "Interface\\Icons\\Spell_Misc_EmotionHappy" 
	elseif db.display_icon == 5 then icon = "Interface\\Icons\\Spell_Misc_EmotionAngry"
	elseif db.display_icon == 6 then icon = "Interface\\Icons\\Spell_Misc_EmotionSad"
	elseif db.display_icon == 7 then icon = "Interface\\TUTORIALFRAME\\TutorialFrame-QuestionMark"
	elseif db.display_icon == 8 then icon = "Interface\\CHARACTERFRAME\\UI-Player-PlayTimeTired"
	elseif db.display_icon == 9 then icon = "Interface\\CHARACTERFRAME\\UI-Player-PlayTimeUnhealthy"
	end

	self.Frame.texture:SetTexture(nil)
	self.Frame.texture:SetTexture(icon)

end

function Emo:SetPosition()

	self.Frame:ClearAllPoints()
	if db.display_x and db.display_y then
		self.Frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", db.display_x, db.display_y)
		self.Frame:SetUserPlaced(nil)
	else
		self.Frame:SetPoint("CENTER", 0, 0)
	end

end

function Emo:Reset()
	Emo:SetPosition()
	Emo:Scale()
	Emo:Alpha()
	Emo:Icon()
	Emo:Show()
end

function Emo:Tooltip()
	local t
	GameTooltip_SetDefaultAnchor(GameTooltip, self.Frame:GetParent())
	GameTooltip:SetText("Emo", 1, 1, 1)
	if db.button_click == "ALL" then 
		t = string.format("Left-Click for Quick Emote List")
	elseif db.button_click then
		t = string.format("Left-Click to /%s", db.button_click)
	end
	GameTooltip:AddLine(t, 0, 1, 0, 1)
	GameTooltip:AddLine("Right-Click for Full Emote List", nil, nil, nil, 1)
	GameTooltip:Show()
end


---------------------------------------------------------------------------------
-- EMOTE FUNCTIONS

function Emo:DoEmote(emote)

	if emote then
		DoEmote(emote)
	end

end


---------------- GAMEOPTIONS MENU ----------------

local emo_options = {

	type = "group",
	name = "Emo",
	get = function(info) return db[ info[#info] ] end,
	set = function(info, value) db[ info[#info] ] = value end,
	args = {
		General = {
			order = 1,
			type = "group",
			name = "General Settings",
			desc = "General Settings",
			args = {
				blank10 = {
					order = 10,
					type = "description",
					name = " ",
				},
				display_show = {
					order = 11,
					name = "Hide/Show Button",
					desc = "Hide/Show the Emo Button.",
					type = "toggle",
					set = function(info, value)
						db.display_show = value
						Emo:Show()
					end,
				},
				display_unlocked = {
					order = 12,
					name = "Lock/Unlock Button",
					desc = "Lock/Unlock the Button.",
					type = "toggle",
					set = function(info, value)
						db.display_unlocked = value
					end,
				},
				blank20 = {
					order = 20,
					type = "description",
					name = " ",
				},
				display_tooltip = {
					order = 21,
					name = "Hide/Show Tooltip",
					desc = "Hide/Show the tooltip on mouseover.",
					type = "toggle",
					set = function(info, value)
						db.display_tooltip = value
					end,
				},
				blank30 = {
					order = 30,
					type = "description",
					name = " ",
				},
				display_scale = {
					order = 31,
					name = "Button Scale",
					desc = "Change the size of the Emo Button.",
					type = "range",
					min = 16, max = 64, step = 1,
					set = function(info, value)
						db.display_scale = value
						Emo:Scale()
					end,
				},
				display_alpha = {
					order = 32,
					name = "Button Alpha",
					desc = "Change the transparency of the Emo Button.",
					type = "range",
					min = 0.20, max = 1, step = 0.01,
					isPercent = true,
					set = function(info, value)
						db.display_alpha = value
						Emo:Alpha()
					end,
				},
				display_icon = {
					type = "select",
					order = 41,
					name = "Icon",
					desc = "Select the Icon to use for Emo.",
					values = {
						[1] = "Chat Button",
						[2] = "Mask 1",
						[3] = "Mask 2",
						[4] = "Happy Face",
						[5] = "Angry Face",
						[6] = "Sad Face",
						[7] = "Chat Bubble",
						[8] = "Happy Face Button",
						[9] = "Angry Face Button",
					},
					set = function(info, value)
						db.display_icon = value
						Emo:Icon()
					end,
				},
				button_click = {
					type = "select",
					order = 42,
					name = "On Left Click",
					desc = "Set the action to perform when left clicking the button.",
					values = {
						ALL = "Show Quick List",
						BOW = "/Bow",
						CHEER = "/Cheer",
						COWER = "/Cower",
						CRY = "/Cry",
						DANCE = "/Dance",
						HAIL = "/Hail",
						HELLO = "/Hello",
						HUG = "/Hug",
						KNEEL = "/Kneel",
						LOVE = "/Love",
						SALUTE = "/Salute",
						WAVE = "/Wave",
					},
					set = function(info, value)
						db.button_click = value
					end,
				},
				blank70 = {
					order = 70,
					type = "description",
					name = " ",
				},
				reset_options = {
					order = 80,
					name = "Default Settings",
					desc = "Reset all settings to default.",
					type = "execute",
					func = function()
						Emo.db:ResetProfile(true)
						Emo:Reset()
					end,
				},
				reset_position = {
					order = 90,
					name = "Default Position",
					desc = "Reset the position of the button to the middle of the screen.",
					type = "execute",
					func = function()
						Emo.Frame:ClearAllPoints()
						Emo.Frame:SetPoint("CENTER",0,0)
						Emo.Frame:Show()
					end,
				},
			},
		},
	},
}

function Emo:SetupOptions()
	self.optionsFrames = {}
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Emo", emo_options)
	self.optionsFrames.Emo = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Emo", nil, nil, "General")
	self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles")

end

function Emo:RegisterModuleOptions(name, optionTbl, displayName)
	emo_options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Emo", displayName, "Emo", name)
end

function Emo:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Emo)
end


---------------------------------------------------------------------------------
-- DEWDROP MENU

do

	local info = {}
	local EmoMenu1= CreateFrame("Frame", "EmoMenu1")
	EmoMenu1.displayMode = "MENU"
	EmoMenu1.autoHideDelay = 6
	EmoMenu1.initialize = function(self, level)
		if not level then return end
		for k in pairs(info) do info[k] = nil end
		if level == 1 then

			info.disabled     = nil
			info.isTitle      = nil
			info.notCheckable = 1

			info.hasArrow 	= false

			info.colorCode = "|c00ffff00"

			info.text         = "/Bow"
			info.func         = function() DoEmote("BOW") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Cheer"
			info.func         = function() DoEmote("CHEER") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Cower"
			info.func         = function() DoEmote("COWER") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Cry"
			info.func         = function() DoEmote("CRY") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Dance"
			info.func         = function() DoEmote("DANCE") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Hail"
			info.func         = function() DoEmote("HAIL") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Hello"
			info.func         = function() DoEmote("HELLO") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Hug"
			info.func         = function() DoEmote("HUG") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Kneel"
			info.func         = function() DoEmote("KNEEL") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Love"
			info.func         = function() DoEmote("LOVE") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Salute"
			info.func         = function() DoEmote("SALUTE") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

			info.text         = "/Wave"
			info.func         = function() DoEmote("WAVE") end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)


			info.colorCode = "|c00ffffff"

			-- Close menu item
			info.text         = "Cancel"
			info.func         = function() CloseDropDownMenus() end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

		end
	end

	local info = {}
	local EmoMenu2= CreateFrame("Frame", "EmoMenu2")
	EmoMenu2.displayMode = "MENU"
	EmoMenu2.autoHideDelay = 6
	EmoMenu2.initialize = function(self, level)
		if not level then return end
		for k in pairs(info) do info[k] = nil end
		if level == 1 then

			info.disabled     = nil
			info.isTitle      = nil
			info.notCheckable = 1

			info.hasArrow 	= true

			info.text         = "Voice & Animation"
			info.checked      = nil
			info.value 		= 1
			UIDropDownMenu_AddButton(info, level)

			info.text         = "Sound & Animation"
			info.checked      = nil
			info.value 		= 2
			UIDropDownMenu_AddButton(info, level)

			info.text         = "Animation Only"
			info.checked      = nil
			info.value 		= 3
			UIDropDownMenu_AddButton(info, level)

			info.text         = "Basic Emotes"
			info.checked      = nil
			info.value 		= 4
			UIDropDownMenu_AddButton(info, level)

			info.hasArrow 	= false

			-- Close menu item
			info.text         = "Cancel"
			info.func         = function() CloseDropDownMenus() end
			info.checked      = nil
			info.arg1         = nil
			UIDropDownMenu_AddButton(info, level)

		end

		if level > 1 then
			if Emotes[UIDROPDOWNMENU_MENU_VALUE] ~= nil and UIDROPDOWNMENU_MENU_VALUE ~= nil then
				for Index, Value in pairs(Emotes[UIDROPDOWNMENU_MENU_VALUE]) do
					if Value[1] == 1 then
						info.text = string.lower(Value[2])
						info.func = function() Emo:DoEmote(Value[2]) end
						info.hasArrow = false
					else
						info.text = Value[2]
						info.value = Value[1]
						info.hasArrow = true
					end
					info.checked = nil
					info.owner = EmoMenu2
					info.icon = nil
					UIDropDownMenu_AddButton(info, level)
				end
			end
		end
	end
end


